package provide Paved::button 1.1

##  pavebutton.tcl
##
##	Paved::button : an extension of the button widget.
##
##  Copyright (c) 2004-2006 <Irrational Numbers> : <aburatti@libero.it> 
##
##  NOTE: package "snit" (v0.91 or later) is required.
##   It is freely downloadable from the Web:
##       http://www.wjduquette.com/snit
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation.
##
##  See the file "license" for information on usage and
##  redistribution of this program, and for a disclaimer of all warranties.
##

package require snit

#
# How to use Paved::button:
#   Read "pavebutton.txt" for detailed info.
#   Sample code in provided in "demo*.tcl".
#


::snit::widgetadaptor Paved::buttonAdaptor {
   
   variable bg
   variable isInternalTile false
   variable srcImage ""

   delegate method * to hull
   
   option -tile      -configuremethod Set_tile
   option -compound  -configuremethod Set_compound -default none
   delegate option * to hull except { -image -compound } 

    #define a 'pseudo' Class binding
   typeconstructor {
      bind PavedButton <Configure> { %W _RedrawBg %w %h }
   }


   constructor {args} {
      installhull $win

      set bg [image create photo ${win}_bg]
      $win configurelist $args

      bindtags $win [linsert [bindtags $win] 1 PavedButton]
   }


   destructor {
      image delete $bg
      $win _RemoveInternalTile
   }
   

    # intercept option -compound.
    # Only  "none" and "center" are valid.
    # Other 'valid' options are forced to "center" 
    #  (just for preserving old clients) 
   method Set_compound {option value} {
       switch -- $value {
           none -
           center  { set value $value }
           top -
           left -
           right -
           bottom  { set value center }

           default { return -code error \
                     "bad compound \"$value\": must be  none, center"
                   }
       }
       $hull configure -compound $value
       set options(-compound) $value
   }


    # Private
   method _RemoveInternalTile {} {
      # INVARIANT  ::  isInternalTile == true  ==>  srcImage != {}
      #                srcImage == {}  ==>  isInternalTile == false
      if { $isInternalTile } { 
           image delete $srcImage
           set isInternalTile false
           set srcImage {}
      }
   }


   method Set_tile {option value} {      
      set errMsg ""
      if { $value != {} } {

           # is it an 'image' or not ?
          if { [catch {image type $value}] } {
               # $value is not an 'image' ; assume it is a filename
              if { [catch {image create photo -file $value} newImage] } {
                  set errMsg $newImage
                   # leave existing srcImage
              } else {                   
                  $win _RemoveInternalTile
                  set isInternalTile true
                  set srcImage $newImage
              }              
          } else {
               # $value is an 'image'
              $win _RemoveInternalTile
              set srcImage $value
          }
          if { $errMsg == "" } { $hull configure -image $bg }
      
      } else {

         $win _RemoveInternalTile         
         set srcImage {}
         $hull configure -image {}
      }
      
      if { $errMsg == "" } {
          set options(-tile) $value
          $win _RedrawBg [winfo width $win] [winfo height $win]
          return
      } else {
          return -code error $errMsg
      }
   }
   

    # Private
   method _RedrawBg { w h } {
      if { $srcImage != {} } {
          # re-create an empty image
         image create photo $bg
          # note that distances may be in different formats
          #  (inches,mm,points,pixels)
	  # convert them in pixels
         set bd   [winfo pixels $win [$hull cget -bd]]
	 set ht   [winfo pixels $win [$hull cget -highlightthickness]]
         set padx [winfo pixels $win [$hull cget -padx]]
         set pady [winfo pixels $win [$hull cget -pady]]

         set w [winfo pixels $win $w]
         set h [winfo pixels $win $h]

         set D  [expr $bd + $ht]
	    
         set w [expr $w -2*($D + $padx)]
	 set h [expr $h -2*($D + $pady)]

         if { $w > 0  && $h > 0 } {
            $bg copy $srcImage  -to 0 0 $w $h
         }
      }
   }

}


namespace eval Paved { ; }

  # this command *creates* a *new* Paved::button widget
  # (in contrast with Paved::buttonAdaptor 
  #  that wraps an *existing* button widget)
proc Paved::button {path args} {
   eval buttonAdaptor [::button $path] $args
   return $path
}

